-- Copyright (C) 2003 Roger Browne. May be freely distributed under
-- the terms of the Eiffel Forum License version 2; see file LICENSE.

class FACTZ_FILE
	-- Reads from the FACTZ file and creates FACT objects.
	-- Writes to the FACTZ file.

feature {ANY}
	-- commands

	append_new_fact(fact_text: STRING) is
		require
			fact_text: fact_text /= void
		local
			file: TEXT_FILE_WRITE
			text: STRING
		do
			create file.connect_for_appending_to(factz_filename)
			put_datestamp(file)
			file.put_new_line
			text := escape(fact_text)
			if text.count > 0 then
				file.put_string(text)
				if text.item(text.count) /= '%N' then
					file.put_new_line
				end
			end
			file.put_string(separator)
			file.put_new_line
			file.disconnect
		ensure
			datestamp: datestamp /= void
		end
		
	delete_fact(fact_date: STRING) is
		require
			fact_date: fact_date /= void and then is_valid_date(fact_date)
		local
			file: TEXT_FILE_WRITE
		do
			create file.connect_for_appending_to(factz_filename)
			put_datestamp(file)
			file.put_string(" deletes ")
			file.put_string(fact_date)
			file.put_new_line
			file.put_string(separator)
			file.put_new_line
			file.disconnect
		ensure
			datestamp: datestamp /= void
		end
		
	modify_fact(fact_date: STRING; fact_text: STRING) is
		require
			fact_date: fact_date /= void and then is_valid_date(fact_date)
			fact_text: fact_text /= void
		local
			file: TEXT_FILE_WRITE
		do
			create file.connect_for_appending_to(factz_filename)
			put_datestamp(file)
			file.put_string(" modifies ")
			file.put_string(fact_date)
			file.put_new_line
			if fact_text.count > 0 then
				file.put_string(fact_text)
				if fact_text.item(fact_text.count) /= '%N' then
					file.put_new_line
				end
			end
			file.put_string(separator)
			file.put_new_line
			file.disconnect
		ensure
			datestamp: datestamp /= void
		end

feature {ANY}
	-- queries
	
	datestamp: STRING  -- time and date of last successful add, modify or delete; void otherwise

	fact(index: INTEGER): FACT is
		require
			valid_index: index >= 1 and index <= fact_count
		do
			read_factz
			result := factz.item(index)
		ensure
			nonvoid: result /= void
		end

	fact_with_date(fact_date: STRING): FACT is
		-- Return the fact with date 'fact_date', or 'void' if no fact has that date.
		require
			fact_date: fact_date /= void and then is_valid_date(fact_date)
		do
			read_factz
			result := factz_table.reference_at(fact_date)
		ensure
			match: result /= void implies result.date.is_equal(fact_date)
			no_match: result = void implies not factz_table.has(fact_date)
		end
	
	fact_count: INTEGER is
		do
			read_factz
			result := factz.count
		ensure
			count: result = factz.count
		end

feature {ANY}
	-- queries (not involving 'current')
	
	is_valid_date(d: STRING): BOOLEAN is
		-- Is 'd' in the valid date format for a FACTZ file?
		-- (This could be made more robust at the expense of increased complexity.)
		require
			d: d /= void
		do
			result := d.count = 19
				and then d.item(1).in_range('2', '9')
				and then d.item(2).is_digit
				and then d.item(3).is_digit
				and then d.item(4).is_digit
				and then d.item(5) = '-'
				and then d.item(6).in_range('0', '1')
				and then d.item(7).is_digit
				and then d.item(8) = '-'
				and then d.item(9).in_range('0', '3')
				and then d.item(10).is_digit
				and then d.item(11) = ' '
				and then d.item(12).in_range('0', '2')
				and then d.item(13).is_digit
				and then d.item(14) = ':'
				and then d.item(15).in_range('0', '5')
				and then d.item(16).is_digit
				and then d.item(17) = ':'
				and then d.item(18).in_range('0', '5')
				and then d.item(19).is_digit
		end
		
feature {NONE}
	-- commands
	
	put_datestamp(file: TEXT_FILE_WRITE) is
		require
			file: file /= void
		do
			datestamp := now
			file.put_string(datestamp)
		ensure
			datestamp: datestamp /= void
		end

	read_factz is
		-- Read the factz file into memory (but only do it once).
		-- TODO: Speed could be improved by using a seekable FILE and reading the end of file first.
		local
			current_date, linked_date: STRING
			current_fact, linked_fact: FACT
			file: TEXT_FILE_READ
			line: STRING
		do
			if factz = void then
				create factz.make(1, 0)
				create factz_table.make
				from
					create file.connect_to(factz_filename)
				until file.end_of_input loop
					file.read_line
					line := file.last_string.twin
					if current_fact = void then
						if line.count = 19 and then is_valid_date(line) then
							create current_fact.make
							current_fact.set_date(line)
						elseif line.count = 47 and then is_valid_date(line.substring(1, 19)) and then line.substring_index(" deletes ", 1) = 20 and then is_valid_date (line.substring(29, 47)) then
							current_date := line.substring(1, 19)
							linked_date := line.substring(29, 47)
							create current_fact.make
							linked_fact := fact_with_date(linked_date)
							if linked_fact = void then
								print("Content-type: text/plain%N%N")
								print("Error 1 in FACTZ_FILE.%N")
							else
								current_fact.set_date(current_date)
								current_fact.set_deletes(linked_fact)
								linked_fact.set_deleted_by(current_fact)
							end
						elseif line.count = 48 and then is_valid_date(line.substring(1, 19)) and then line.substring_index(" modifies ", 1) = 20 and then is_valid_date(line.substring(30, 48)) then
							current_date := line.substring(1, 19)
							linked_date := line.substring(30, 48)
							create current_fact.make
							linked_fact := fact_with_date(linked_date)
							if linked_fact = void then
								print("Content-type: text/plain%N%N")
								print("Error 2 in FACTZ_FILE.%N")
							else
								current_fact.set_date(current_date)
								current_fact.set_modifies(linked_fact)
								linked_fact.set_modified_by(current_fact)
							end
						else
							-- error in file format (or EOF?)
						end
					elseif line.is_equal(separator) then
						factz.force(current_fact, factz.upper + 1)
						factz_table.add(current_fact, current_fact.date)
						current_fact := void
					else
						current_fact.add_line(line)
					end
				end
				file.disconnect
			end
		end

feature {NONE}
        -- queries (not involving 'current')

	as_two_digits(i: INTEGER): STRING is
		-- 'i' expressed using two digits, with leading zero if necessary
		require
			i: i >= 0 and i <= 99
		do
			result := i.out
			if result.count = 1 then
				result.insert_character('0', 1)
			end
		ensure
			length: result.count = 2
			value: result.to_integer = i
		end
	
	escape(text: STRING): STRING is
		-- Escape any embedded separators with a trailing space (if followed by a control character)
		require
			text: text /= void
		local
			i: INTEGER
		do
			result := text.twin
			from
				i := 1
			variant
				result.count - i + 1
			until i = result.count + 1 loop
				i := result.substring_index(separator, i)
				if i = 0 then
					i := result.count + 1
				else
					i := i + separator.count
					if i = result.count + 1 or else ("%R%N").has(result.item(i)) then
						result.insert_character(' ', i)  -- escape the separator string
						i := i + 1
					end
				end
			end
		ensure
			result.count >= text.count
		end

	factz_filename: STRING is "factz"  -- file will be created in same directory as cgi application

	separator: STRING is "~~~~"

	now: STRING is
		-- Current time in format YYYY-MM-DD HH:MM:SS
		local
			timestamp: TIME
		do
			timestamp.update
			create result.make(19)
			result.append_string(timestamp.year.out)
			result.append_character('-')
			result.append_string(as_two_digits(timestamp.month))
			result.append_character('-')
			result.append_string(as_two_digits(timestamp.day))
			result.append_character(' ')
			result.append_string(as_two_digits(timestamp.hour))
			result.append_character(':')
			result.append_string(as_two_digits(timestamp.minute))
			result.append_character(':')
			result.append_string(as_two_digits(timestamp.second))
		ensure
			valid: is_valid_date(result)
		end

feature {NONE}
	-- queries (basic specifiers)
	
	factz: ARRAY[FACT]  -- the factz read from the file
	
	factz_table: DICTIONARY[FACT, STRING]
	
invariant

	in_step: (factz = void) = (factz_table = void)
	elements_in_both: factz /= void implies factz.count = factz_table.count

end
