-- Copyright (C) 2003 Roger Browne. May be freely distributed under
-- the terms of the Eiffel Forum License version 2; see file LICENSE.

class WEBPAGE
	-- Accumulate items that are to be displayed on the webpage, then
	-- add HTML markup and print the HTML page to standard output.
	
	-- USAGE:
	-- 1. Call 'make' to create the object.
	-- 2. Call the 'set_*' and 'add_*' commands to record the items that are to be displayed.
	-- 3. Call one of the 'emit_*' commands to produce an HTML page in the specified format.

creation {ANY} make

feature {NONE}
	-- creation
	
	make is
		do
			create list_items.make(1, 0)
		ensure
			empty_list_items: list_items.count = 0
		end

feature {ANY}
	-- commands

	emit_confirm_delete_page is
		-- Ask the user to confirm deletion of the selected fact
		do
			html_prologue
			generate_page_header
			start_division("left")
			generate_new_fact_form
			generate_delete_confirmation
			end_division
			generate_search_and_list
			html_epilogue
		end

	emit_error_page is
		-- Display an error message and offer the user to proceed to the 'new fact' page.
		do
			html_prologue
			generate_page_header
			generate_error_message
			html_epilogue
		end

	emit_modify_fact_page is
		-- Allow the user to modify the selected fact
		do
			html_prologue
			generate_page_header
			start_division("left")
			generate_new_fact_form
			generate_modify_fact_box
			end_division
			generate_search_and_list
			html_epilogue
		end
	
	emit_new_fact_page is
		do
			html_prologue
			generate_page_header
			start_division("left")
			generate_new_fact_form
			end_division
			generate_search_and_list
			html_epilogue
		end

	emit_view_fact_page is
		do
			html_prologue
			generate_page_header
			start_division("left")
			generate_new_fact_form
			generate_view_fact_box
			end_division
			generate_search_and_list
			html_epilogue
		end

	set_error_message(em: STRING) is
		require
			em: em /= void and then not em.is_empty
		do
			error_message := em
		ensure
			set: error_message = em
		end
		

	set_fact(f: FACT) is
		require
			nonvoid_f: f /= void
		do
			fact := f
		ensure
			set: fact = f
		end
		
	set_searchtext(st: STRING) is
		do
			searchtext := st
		ensure
			set: searchtext = st
		end
		
	add_list_item(f: FACT) is
		-- Add a summary of the fact as a list item, with a hyperlink to enable the full fact to be displayed.
		require
			nonvoid_f: f /= void
		do
			list_items.force(f, list_items.upper + 1)
		ensure
			extended: list_items.count = 1 + old list_items.count
			added: list_items.item(list_items.upper) = f
		end

feature {NONE}
	-- commands (HTML generation)

	end_division is
		do
			print("</div>%N")
		end
	
	generate_delete_confirmation is
		do
			print("<div class='confirm'>")
			print("<h2>Confirm deletion</h2>%N")
			generate_date_and_factbox(true)
			print("<form method='get' action='factz.cgi'>%N")
			print("<p>Do you want to delete this item?%N")
			print("<input type='submit' name='Confirmed' value='Yes' />%N")
			print("<input type='submit' name='Confirmed' value='No' />%N")
			print("<input type='hidden' name='submit' value='Delete' />%N")
			generate_hidden_searchtext
			generate_hidden_factdate
			print("</p></form>%N")
			print("</div>%N")
		end
	
	generate_date_and_factbox(readonly: BOOLEAN) is
		-- Display fact date plus fact contents for viewing or modification
		do
			if fact = void then
				print("<p>(Date is missing or invalid!)</p>%N")
			else
				print("<p>")
				print(fact.date)
				print("</p>%N")
				print("<p><textarea name='Facttext' rows='10' cols='80' ")
				if readonly then
					print("readonly='readonly' class='readonly'>")
				else
					print("class='stretch'>")
				end
				generate_fact_text
				print("</textarea></p>%N")
			end
		end
	
	generate_error_message is
		do
			print("<div class='error'>%N")
			print("<h2>Error</h2>%N")
			print("<form method='get' action='factz.cgi'>%N")
			print("<p>")
			if error_message /= void then
				print(error_message)
			else
				print("No error message is available.")
			end
			print("</p>%N")
			print("<p><input type='submit' value='Return to main page' /></p>%N")
			print("</form>%N")
			print("</div>%N")
		end

	generate_fact_text is
		-- Output the text of the fact (e.g. to fill a textbox)
		local
			i: INTEGER
		do
			from
				i := 0
			invariant -- 'i' lines of the fact have been output
			variant fact.line_count - i
			until i = fact.line_count loop
				i := i + 1
				print(fact.line(i))
				print('%N')
			end
		end
	
	generate_hidden_factdate is
		-- Generate a hidden field containing the current fact date (to persist it from page to page)
		do
			print("<input type='hidden' name='Factdate' value='")
			print(fact.date)
			print("' />%N")
		end

	generate_hidden_searchtext is
		-- Generate a hidden field containing the search text - if nonvoid (to persist it from page to page)
		do
			if searchtext /= void and then not searchtext.is_empty then
				print("<input type='hidden' name='Searchtext' value='")
				print(searchtext)
				print("' />%N")
			end
		end
	
	generate_list is
		local
			i: INTEGER
			current_fact: FACT
		do
			print("<div class='list'>%N")
			print("<h2>")
			if searchtext = void or else searchtext.is_empty then
				print("FACTZ:")
			else
				print("Search results:")
			end
			print("</h2><ul class='rollover'>")
			from
				i := list_items.lower
			until i > list_items.upper loop
				current_fact := list_items.item(i)
				print("<li>%N<a href='factz.cgi?submit=View")
				if searchtext /= void and then not searchtext.is_empty then
					print("&amp;Searchtext=")
					print(searchtext)
				end
				print("&amp;Fact=")
				print(current_fact.date)
				print("'><i><b>")
				print(current_fact.date)
				print("</b></i><br />%N")
				summarise_fact(current_fact)
				print("</a></li>")
			-- advance the loop
				i := i + 1
			end
			print("</ul></div>%N")
		end

	generate_modify_fact_box is
		-- Display the fact for editing
		do
			print("<div class='modify'>")
			print("<h2>Modify selected FACT:</h2>%N")
			print("<form method='get' action='factz.cgi'>%N")
			generate_date_and_factbox(false)
			print("<p><input type='submit' name='Confirmed' value='Cancel' /> ")
			print("<input type='reset' value='Reset' /> ")
			print("<input type='submit' name='Confirmed' value='Save' /> ")
			generate_hidden_searchtext
			generate_hidden_factdate
			print("<input type='hidden' name='submit' value='Modify' /></p>%N")
			print("</form>")
			print("</div>")
		end
	
	generate_new_fact_form is
		do
			print("<div class='new-fact-form'>%N")
			print("<h2>Enter new FACT:</h2>%N")
			print("<form method='get' action='factz.cgi'>%N")
			print("<p><textarea class='stretch' name='Facttext' rows='10' cols='80'></textarea></p>%N")
			print("<p><input type='reset' value='Clear' /> <input type='submit' name='submit' value='Save' /></p>%N")
			print("</form>%N")
			print("</div>%N")
		end
		
	generate_page_header is
		do
			print("<div class='page-header'>%N")
			print("<h1>FACTZ Personal Information Store</h1>%N")
			print("<p><b>By Roger Browne</b> - Version 0.9</p>%N")
			print("</div>%N")
		end

	generate_search_and_list is
		do
			start_division("right")
			generate_search_form
			generate_list
			end_division
		end
	
	generate_search_form is
		do
			print("<div class='search-form'>%N")
			print("<h2>Search for FACTZ:</h2>%N")
			print("<form method='get' action='factz.cgi'>%N")
			print("<p><input class='stretch' type='text' name='Searchtext' size='30' ")
			if searchtext /= void and then not searchtext.is_empty then
				print("value='")
				print(searchtext)
				print("' ")
			end
			print("/></p><p><input type='submit' name='submit' value='Search' /> ")
			print("<input type='submit' name='submit' value='Show All' /></p>%N")
			print("</form>%N")
			print("</div>%N")
		end
	
	generate_view_fact_box is
		-- display fact with date 'fact_date', together with heading and modify/delete buttons
		do
			print("<div class='fact'>")
			print("<h2>Selected FACT:</h2>%N")
			generate_date_and_factbox(true)
			print("<form method='get' action='factz.cgi'>%N")
			print("<p><input type='submit' name='submit' value='Modify' /> ")
			print("<input type='submit' name='submit' value='Delete' />%N")
			generate_hidden_searchtext
			generate_hidden_factdate
			print("</p></form>%N")
			print("</div>")
		end

	html_epilogue is
		do
			print("</body></html>%N")
		end
		
	html_prologue is
		do
			print("Content-type: text/html%N%N")
			print("<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd'>%N")
			print("<html xmlns='http://www.w3.org/1999/xhtml'>%N")
			print("<head><meta http-equiv='content-type' content='text/html; charset=utf-8' />%N")
			print("<title>FACTZ Personal Information Store</title>%N")
			print("<link rel='stylesheet' type='text/css' href='factz.css' title='two-column' />%N")
			print("</head><body>%N")
		end

	start_division(division_name: STRING) is
		require
			division_name: division_name /= void and then not division_name.is_empty
		do
			print("<div class='")
			print(division_name)
			print("'>%N")
		end
	
	summarise_fact(f: FACT) is
		-- Display a summary of the fact.
		require
			nonvoid_f: f /= void
		local
			couldnt_all_fit: BOOLEAN
			desired_length: INTEGER
			end_position: INTEGER
			line: STRING
			line_index: INTEGER
		do
			from
				line_index := 0
				desired_length := summary_length
			invariant -- 'line_index' lines have been output
				desired_length: desired_length.in_range(0, summary_length)
				line_index: line_index.in_range(0, f.line_count)
			variant f.line_count - line_index
			until
				line_index = f.line_count or desired_length <= 1  -- no point just adding a space
			loop
				line_index := line_index + 1
				line := f.line(line_index)
				if not line.is_empty then
					if line_index > 1 then
						desired_length := desired_length - 1  -- allow for a leading space
					end
					if line.count <= desired_length then
						if line_index > 1 then
							print(' ')
						end
						print(line)
						desired_length := desired_length - line.count
					else  -- Find a word boundary
						couldnt_all_fit := true
						from
							end_position := desired_length
						invariant
							end_position: end_position.in_range(0, line.count - 1)
						variant
							end_position
						until end_position = 0 or else line.item(end_position).is_separator loop
							end_position := end_position - 1
						end
						if end_position < desired_length - 20 then
							-- truncate very long words instead of omitting them
							end_position := desired_length
						end
						if end_position > 0 then
							if line_index > 1 then
								print(' ')
							end
							print(line.substring(1, end_position))
						end
						desired_length := 0
					end
				end
			end
			if couldnt_all_fit or else line_index < f.line_count then
				print("...")
			end
		end

feature {NONE}
	-- queries

	error_message: STRING
		-- Text of error message to be displayed
		
	fact: FACT
		-- Current fact for display, modification or delete confirmation
		
	list_items: ARRAY[FACT]
		-- Items from summary list of factz displayed
	
	searchtext: STRING
		-- Search text to populate "Search" box. Can be 'void' if all items are to be displayed.

	summary_length: INTEGER is 200
		-- How many characters to use for the summary
		
invariant

	nonvoid_list_items: list_items /= void

end
